#
# Copyright (c) 2015 Midokura SARL, All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser
    ConfigParser.SafeConfigParser = ConfigParser.ConfigParser
import os

from charmhelpers.contrib.openstack import utils
from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core.services import base
from charmhelpers.core.services import helpers
from midonet_helpers import puppet
from midonet_helpers import relations as common_relations
from midonet_helpers import repositories
import relations

ROOTWRAP_PATH = '/etc/nova/rootwrap.d'
FAKE_HEADER = 'faaake'


def manage():
    cfg = hookenv.config()
    hookenv.status_set('maintenance', 'reconfiguring charm')
    ost_origin = cfg.get('openstack-origin')
    midonet_origin = cfg.get('midonet-origin')
    mem_username = cfg.get('mem-username')
    mem_pwd = cfg.get('mem-password')
    if None in (ost_origin, midonet_origin) or (
            midonet_origin is not None and midonet_origin[:3] == 'mem' and None
            in (mem_username, mem_pwd)):
        hookenv.status_set('maintenance', 'lacking repo configuration')
        return

    ost_release = utils.get_os_codename_install_source(ost_origin)
    repo = repositories.config('%s/%s' % (ost_release, midonet_origin),
                               mem_username, mem_pwd)
    manager = base.ServiceManager([
        {
            'service': 'midolman',
            'ports': [],
            'provided_data': [],
            'required_data': [relations.CassandraRelation(),
                              common_relations.ZookeeperRelation(),
                              repo],
            'data_ready': [
                _write_rootwrap,
                helpers.render_template(
                    source='common.yaml',
                    target='/var/lib/hiera/midonet_agent/common.yaml'),
                puppet.PuppetCallback(
                    actions="""class {'midonet::midonet_agent':}""",
                    ordering="""Class['midonet::midonet_agent::install']""",
                    ctxt={}),
                HostRelationHandling(),
            ],
        },
    ])
    manager.manage()
    if host.service_running('midolman'):
        hookenv.status_set('active', 'midonet agent up and runing')


def _write_rootwrap(*args, **kwargs):
    """Write MidoNet's rootwrap when running on  a compute node."""
    if os.path.exists(ROOTWRAP_PATH):
        with open(os.path.join(ROOTWRAP_PATH, 'midonet.filters'), 'w') as filt:
            filt.write('[Filters]\nmm-ctl: CommandFilter, mm-ctl, root')


class HostRelationHandling(base.ManagerCallback):
    """Host relation data sending

    Due to the nature of some data, that depends on the agent service having
    run, it needs to send the data after the puppet installation callback
    """

    def host_uuid(self):
        id_location = '/etc/midonet_host_id.properties'
        legacy_id_location = '/etc/midolman/host_uuid.properties'
        config = ConfigParser.SafeConfigParser()
        try:
            config.readfp(PropertyWrapper(open(id_location)))
            host_uuid = config.get(FAKE_HEADER, 'host_uuid')
        except Exception as ex:
            hookenv.log('Failed to read host uuid from %s. Err %s' %
                        (id_location, ex), level=hookenv.WARNING)
            try:
                config.readfp(PropertyWrapper(open(legacy_id_location)))
                host_uuid = config.get(FAKE_HEADER, 'host_uuid')
            except Exception as ex:
                hookenv.log('Failed to read host uuid from %s. Err %s' %
                            (legacy_id_location, ex), level=hookenv.WARNING)
                host_uuid = None
        return host_uuid

    def __call__(self, manager, service_name, event_name):
        host_uuid = self.host_uuid()
        if host_uuid is None:
            hookenv.status_set('maintenance', 'lacking uuid configuration')
            return
        for rid in hookenv.relation_ids('host'):
            hookenv.relation_set(host_uuid=host_uuid, relation_id=rid)


class PropertyWrapper(object):
    def __init__(self, fp):
        self._fp = fp
        self._header = '[%s]\n' % FAKE_HEADER

    def readline(self):
        if self._header:
            try:
                return self._header
            finally:
                self._header = None
        else:
            return self._fp.readline()
