#!/usr/bin/env python
# vim: tabstop=4 shiftwidth=4 softtabstop=4 filetype=python
#
# Copyright (c) 2015 Midokura Europe SARL, All Rights Reserved.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import functools
import shutil
import tempfile

import mock
import services

from .midonet_helpers import test

HOOKENV_PATCHES = [
    'charm_dir',
    'config',
    'is_relation_made',
    'log',
    'open_port',
    'relation_get',
    'relation_ids',
    'related_units',
    'relation_set',
    'remote_unit',
    'unit_get']


class HookTests(test.CharmTestCase):
    def setUp(self):
        test.CharmTestCase.setUp(self, services.base.hookenv, HOOKENV_PATCHES)

        def lsb_release():
            return {
                'DISTRIB_ID': 'Ubuntu',
                'DISTRIB_RELEASE': '14.04',
                'DISTRIB_CODENAME': 'trusty',
                'DISTRIB_DESCRIPTION': "Ubuntu 14.04.3 LTS",
            }

        def get_os_codename_package(pkg, fatal=True):
            return 'liberty'

        self.patch_all(services.utils,
                       ('get_os_codename_package',
                        'lsb_release',))
        self.lsb_release.side_effect = lsb_release
        self.get_os_codename_package.side_effect = get_os_codename_package

        _templating = mock.patch.object(services.helpers, 'render_template')
        setattr(self, 'render_template', _templating.start())
        self.addCleanup(_templating.stop)

        _check_call = mock.patch.object(services.base.hookenv.subprocess,
                                        'check_call')
        setattr(self, 'check_call', _check_call.start())
        self.addCleanup(_check_call.stop)

        _puppet = mock.patch.object(services.puppet, '_apply')
        setattr(self, 'apply', _puppet.start())
        self.addCleanup(_puppet.stop)

        self.test_config.config['openstack-origin'] = 'cloud:trusty-liberty'
        self.config.side_effect = self.test_config.get
        self.relation_get.side_effect = self.test_relation.get

        self.charm_dir.return_value = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.charm_dir.return_value)
        test.CharmTestCase.tearDown(self)

    @mock.patch('charmhelpers.core.services.base.service_restart')
    @mock.patch('charmhelpers.core.services.base.service_stop')
    def test_config_change(self, service_stop, service_restart):
        self.data = self._full_relation_data()

        self.relation_ids.side_effect = functools.partial(
            HookTests._full_rids, self)
        self.related_units.side_effect = functools.partial(
            HookTests._related_units, self)
        self.relation_get.side_effect = functools.partial(
            HookTests._relation_get_full, self)

        services.manage()

        self.assertTrue(self.render_template.called)
        self.assertTrue(self.apply.called)
        self.assertTrue(service_restart.called)

    def _full_rids(self, name=None):
        """Returns a random id for each of the 4 relations it establishes."""
        if name is None:
            return self.data.keys()
        rids = []
        for key in self.data.keys():
            if key.startswith(name) and key[len(name)] == ':':
                rids.append(key)
        return rids

    def _related_units(self, rid=None):
        if rid is None:
            return [unit for unit in self.data.values()]
        return self.data[rid].keys()

    def _full_relation_data(self):
        return {
            'cassandra:4': {
                'cassandra/0': {
                    'private-address': 'cass1',
                    'port': 9160
                },
                'cassandra/1': {
                    'private-address': 'cass2',
                    'port': 9160
                },
                'cassandra/2': {
                    'private-address': 'cass3',
                    'port': 9160
                },
                'cassandra/3': {
                    'private-address': 'cass4',
                    'port': 9160
                },
                'cassandra/4': {
                    'private-address': 'cass5',
                    'port': 9160
                },
            },
            'zookeeper:6': {
                'zookeeper/0': {
                    'private-address': 'zoo1',
                    'port': 2181
                },
                'zookeeper/1': {
                    'private-address': 'zoo2',
                    'port': 2181
                },
                'zookeeper/2': {
                    'private-address': 'zoo3',
                    'port': 2181
                },
            },
        }

    def _relation_get_full(self, rid=None, unit=None):
        return self.data[rid][unit]
